# Copyright 2016 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Modified 2017 by Abhi @ Airsource Ltd.
# I've modified the original code to use the webapp2 framework
# Using webapp2 framework for handling/routing requests instead of flask framework
# To see original code, clone repository https://github.com/GoogleCloudPlatform/python-docs-samples.git
# then navigate to python-docs-samples/appengine/standard/firebase/firenotes/backend directory


# [START app]
import json
import google.auth.transport.requests
import google.oauth2.id_token
import requests_toolbelt.adapters.appengine
import webapp2
import ast


from google.appengine.ext import ndb
from datetime import date, datetime

# Use the App Engine Requests adapter. This makes sure that Requests uses
# URLFetch.
requests_toolbelt.adapters.appengine.monkeypatch()
HTTP_REQUEST = google.auth.transport.requests.Request()

# [START note]
class Note(ndb.Model):
    """NDB model class for a user's note.

    Key is user id from decrypted token.
    """
    friendly_id = ndb.StringProperty()
    message = ndb.TextProperty()
    created = ndb.DateTimeProperty(auto_now_add=True)
# [END note]


# [START query_database]
def query_database(user_id):
    """Fetches all notes associated with user_id.

    Notes are ordered them by date created, with most recent note added
    first.
    """
    ancestor_key = ndb.Key(Note, user_id)
    query = Note.query(ancestor=ancestor_key).order(-Note.created)
    notes = query.fetch()

    note_messages = []

    for note in notes:
        note_messages.append({
            'friendly_id': note.friendly_id,
            'message': note.message,
            'created': note.created
        })

    return note_messages
# [END query_database]

def convert_datetime_to_string(datetime_field):
    datetime_string = datetime_field.isoformat()
    return datetime_string

class NotesHandler(webapp2.RequestHandler):
    def get(self):
        """Returns a list of notes added by the current Firebase user."""

        # Verify Firebase auth.
        # [START verify_token]
        id_token = self.request.headers['Authorization'].split(' ').pop()
        claims = google.oauth2.id_token.verify_firebase_token(
            id_token, HTTP_REQUEST)
        if not claims:
            return 'Unauthorized', 401
        # [END verify_token]

        notes = query_database(claims['sub'])


        for n in notes:
            if(n['created'] != None and isinstance(n['created'],(datetime,date))):
                n['created'] = convert_datetime_to_string(n['created'])

        print(notes)
        valid_json = json.dumps(notes,indent=2)
        print(valid_json)


        self.response.headers['Content-Type'] = 'application/json'
        self.response.write(valid_json)
        #print(self.response)






    def post(self):
        """
        Adds a note to the user's notebook. The request should be in this format:
            {
                "message": "note message."
            }
        """
        # Verify Firebase auth.
        id_token = self.request.headers['Authorization'].split(' ').pop()
        claims = google.oauth2.id_token.verify_firebase_token(
            id_token, HTTP_REQUEST)
        if not claims:
            return 'Unauthorized', 401

        print(self.request.body)
        # [START create_entity]
        data = json.loads(self.request.body)

        # Populates note properties according to the model,
        # with the user ID as the key name.
        note = Note(
            parent=ndb.Key(Note, claims['sub']),
            message=data['message'])

        # Some providers do not provide one of these so either can be used.
        note.friendly_id = claims.get('name', claims.get('email', 'Unknown'))
        # [END create_entity]

        # Stores note in database.
        note.put()
        self.response.headers['Content-Type'] = 'application/json'
        self.response.write("{OK}")




    def handle_exception(self, exception,debug_mode):
        webapp2.RequestHandler.handle_exception(self, exception, debug_mode)
# [END app]
